//
//  Galaxy.m

#import "Galaxy.h"
#import "Constants.h"

@implementation Galaxy

- (NSString *)countryCode {
	return @"AU";
}
- (NSString *)languageCode {
	return @"en";
}

/*
 // Limit a search to certain values
 - (NSArray *)limits {
 return [NSArray arrayWithObjects:@"none", @"title", nil];
 }
 // Different search groups
 - (NSArray *)types {
    return nil;
 }
 // Different sorting options
 - (NSArray *)sorts {
 return [NSArray arrayWithObjects:@"Relevance", @"Title", nil];
 }
*/


- (void)searchForString:(NSString *)aString limit:(NSInteger)aLimitNumber type:(NSInteger)typeNumber sort:(NSInteger)sortNumber sender:(id)sender { 	
	
	self.delegate = sender;
	[resultsTitles removeAllObjects];
	[resultsURLs  removeAllObjects];

	aString = [aString stringByReplacingOccurrencesOfString:@" " withString:@"+"];
	NSString *encodedString = [aString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    NSString *searchURL = [NSString stringWithFormat:@"http://www.galaxybooks.com.au/shop/productSearch.do?authorName=&title=&isbn=%@&all=%@&keywords=&sort=&txtQuery=%@", encodedString, encodedString, encodedString];
   
    //NSLog(@"Search URL: %@", searchURL);
	
	[self downloadWebPage:searchURL]; //A static URL for the Galaxy	
}


// Get more info command
- (void)searchFor:(NSDictionary *)searchDict sender:(id)sender {
	
	if ([searchDict objectForKey:@"keyword"]) { //Backward compatability
		NSString *searchTerm = [searchDict objectForKey:@"keyword"];
        [self searchForString:searchTerm limit:0 type:0 sort:0 sender:sender];
	}
/*	else if ([searchDict objectForKey:@"url"] 
			 && [[searchDict objectForKey:@"url"] rangeOfString:@"imdb.com/title/tt"].location != NSNotFound) 	{
        NSString *imdbNumber = [searchDict objectForKey:@"url"];
        NSRange aRange = [imdbNumber rangeOfString:@"imdb.com/title/tt"];
        if ([imdbNumber length] >= aRange.location + aRange.length + 7) { //Check that the URL is long enough
            imdbNumber = [imdbNumber substringWithRange:NSMakeRange(aRange.location + aRange.length, 7)];
            [self searchForString:imdbNumber limit:MKLimitIMDb type:0 sort:0 sender:sender];
        }
	}    */
	else if ([searchDict objectForKey:@"title"] != nil) {
		NSString *searchTerm = [searchDict objectForKey:@"title"];
        [self searchForString:searchTerm limit:0 type:0 sort:0 sender:sender];
	}
	else {
		[sender searchReturnedNumberOfResults:0 sender:self];
	}
}


- (void)downloadComplete:(NSString *)HTMLSource {
	    
    NSArray *resultsSection = [HTMLSource stringsBetween:@"description_cat\"" and:@"</h3>"];
	for (NSString *nextMovie in resultsSection) {
        nextMovie = [nextMovie stringBetween:@"<a " and:@"</a"];
		NSString *url = [nextMovie stringBetween:@"href=\"" and:@"\""];
		NSString *title = [[nextMovie stringBetween:@">" and:@"<"] stringByCleaningHTML];
		
		if (url && title) {
            url = [@"http://www.galaxybooks.com.au" stringByAppendingString:url]; //Missing domain
			[resultsTitles addObject:[title stringByCleaningHTML]];
			[resultsURLs addObject:url];
		}
	}
    
    [delegate searchReturnedNumberOfResults:[resultsURLs count] sender:self];
}


- (NSDictionary *)resultNumber:(int)number {
	//Get the details for one of the results and return it as a dictionary
	
	NSMutableDictionary *resultsDict = [NSMutableDictionary dictionary];
	
	if (number >= [resultsURLs count]) {
		NSLog(@"Error: Galaxy requested out of range: %d of %d", number, [resultsURLs count]);
		[resultsDict setObject:@"No data found on Galaxy" forKey:@"title"];
		return resultsDict;	
	}
	
	//This would be the real url to use, the one requested from the stored in resultsURLs; But changed for a static one as an example
	//NSString *detailsURL = [resultsURLs objectAtIndex:number];
	NSString *detailsURL = [resultsURLs objectAtIndex:number];
	
	NSURLRequest *theRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:detailsURL] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:30.0];
	
	NSString *results = [[[NSString alloc] initWithData:[NSURLConnection sendSynchronousRequest:theRequest returningResponse:nil error:nil] encoding:NSUTF8StringEncoding] autorelease];
	
	if (results == nil)
		return nil;
	
	//NSLog(@"%@", detailsURL);
     [resultsDict setObject:detailsURL forKey:@"url"];
     [resultsDict setValue:[self countryCode] forKey:@"locale"];
     [resultsDict setValue:[self languageCode] forKey:@"localeLanguage"];
	
	NSString *title = [resultsTitles objectAtIndex:number];
	if (title)
		[resultsDict setObject:title forKey:@"title"];
	
	//Get the author
	NSString *tempString = [results stringBetween:@">" and:@"<" startingFrom:@";searchBy=author"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookAuthor];
	
	//The main image
	NSString *imageURL = [results stringBetween:@"class=\"thumb_holder\"" and:@"/>"];
	imageURL = [imageURL stringBetween:@"src=\"" and:@"\""];
	if (imageURL) {
		imageURL = [NSString stringWithFormat:@"http://www.galaxybooks.com.au%@", imageURL];
		[resultsDict setObject:imageURL forKey:MKKeyEntryImageLocation];
	}
    
    //Get the summary
	tempString = [results stringBetween:@">" and:@"</div" startingFrom:@"\"fragment-1\""];
	if (tempString)
		[resultsDict setObject:[[tempString stringByCleaningHTML] stringByCleaningHTML] forKey:MKKeyEntrySummary];

    //Get the Publisher, the others should be easy to duplicate here for details
	tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Imprint:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookPublisher];

    //Get the Dimensions
	tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Dimensions:<"];
	if (tempString) {
        tempString = [tempString stringByCleaningHTML];
        
        NSMutableString *removeReturns = [NSMutableString stringWithString:tempString];
        [removeReturns replaceOccurrencesOfString:@"\n" withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [removeReturns length])];
        [removeReturns replaceOccurrencesOfString:@"\r" withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [removeReturns length])];
        [removeReturns replaceOccurrencesOfString:@"\t" withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [removeReturns length])];
        
        //This is quite a trick, it replaces double spaces for single spaces until it can't do that anymore
        while ([removeReturns replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [removeReturns length])]);
		
        [resultsDict setObject:removeReturns forKey:MKKeyBookDimensions];
    }
    
    //Get the Series
	tempString = [results stringBetween:@"\">" and:@"<" startingFrom:@"Series:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookSeries];   
    
    //Get the number of Pages
	tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Pages:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookPages];
    
    //Get the Format
	tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Format:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookFormat];
    
    //Get the ISBN
	tempString = [results stringBetween:@"\">" and:@"<" startingFrom:@"ISBN:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookISBN];

    //Get the Volume
    tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Volume:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:@"volume"];
    
    //Get the Release Date
    tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Publication Date:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyEntryRelease];

    //Get the Children Level
    tempString = [results stringBetween:@">" and:@"<" startingFrom:@"Audience:<"];
	if (tempString)
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyBookChildrenReadingLevel];
    
    //Get the Price
    tempString = [results stringBetween:@"<h2>" and:@"</h2>" startingFrom:@"showHideSendToFriend"];
    if (tempString) 
        [resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyEntryListPrice];  

    //Get the Genre
    tempString = [results stringBetween:@"<h4>" and:@"</h4>" startingFrom:@"showHideSendToFriend"];
	if (tempString) {
        tempString = [tempString stringByReplacingOccurrencesOfString:@"&nbsp;&raquo;&nbsp;" withString:@", "];
        tempString = [tempString stringByReplacingOccurrencesOfString:@";&nbsp;" withString:@", "];
        NSArray *removeDuplicates = [tempString componentsSeparatedByString:@", "];
        NSSet *aSet = [NSSet setWithArray:removeDuplicates];
        removeDuplicates = [aSet allObjects];
        tempString = [removeDuplicates componentsJoinedByString:@", "];
		[resultsDict setObject:[tempString stringByCleaningHTML] forKey:MKKeyEntryGenre];
    }
    
    return resultsDict;

}

@end
